	##
	##== synthesis version of startup code ===========================
	##
	##   simple startup code for synthesis
	##

	.include "cMIPS.s"
	.text
	.set noreorder
	.align 2
	.extern main
	.global _start,_exit,exit
	.global _excp_0000, _excp_0100, _excp_0180, _excp_0200, _excp_BFC0

        .set MMU_WIRED,    2  ### do not change mapping for base of ROM, I/O
	
	.org x_INST_BASE_ADDR,0
	.ent _start

        ##
        ## reset leaves processor in kernel mode, all else disabled
        ##
_start:	nop

        # get physical page number for 2 pages at the bottom of RAM, for .data
        #  needed so systems without a page table will not break
        #  read TLB[4] and write it to TLB[2]
        li    $k0, 4
        mtc0  $k0, cop0_Index
        ehb
        tlbr
        li    $k1, 2
        mtc0  $k1, cop0_Index
        ehb
        tlbwi


        #  then set another mapping onto TLB[4], to avoid replicated entries
        li    $a0, ( (x_DATA_BASE_ADDR + 8*4096) >>12 )
        sll   $a2, $a0, 12      # tag for RAM[8,9] double-page
        mtc0  $a2, cop0_EntryHi

        li    $a0, ((x_DATA_BASE_ADDR + 8*4096) >>12 )
        sll   $a1, $a0, 6       # RAM[8] (even)
        ori   $a1, $a1, 0b00000000000000000000000000000111 # ccc=0, d,v,g1
        mtc0  $a1, cop0_EntryLo0

        li    $a0, ( (x_DATA_BASE_ADDR + 9*4096) >>12 )
        sll   $a1, $a0, 6       # RAM[9] (odd)
        ori   $a1, $a1, 0b00000000000000000000000000000111 # ccc=0, d,v,g1
        mtc0  $a1, cop0_EntryLo1

        # and write it to TLB[4]
        li    $k0, 4
        mtc0  $k0, cop0_Index
        tlbwi 



        # pin down first four TLB entries: ROM[0], RAM[0], stack and I/O
        li   $k0, 4
        mtc0 $k0, cop0_Wired


        # initialize SP at top of RAM: ramTop - 16
        li   $sp, ((x_DATA_BASE_ADDR+x_DATA_MEM_SZ) - 16)

        # set STATUS, cop0, hw interrupt IRQ7,IRQ6,IRQ5 enabled, user mode
        li   $k0, 0x1000e011
        mtc0 $k0, cop0_STATUS

	
        j main
        nop

	##
	## signal normal program ending
	##
exit:	
_exit:	la   $k0, HW_dsp7seg_addr  	# 7 segment display
	li   $k1, 0x0311		# display .1.1
	sw   $k1, 0($k0)		# write to 7 segment display

	j exit	  # wait forever
	nop
	.end _start

	
        ##
        ##================================================================
        ## exception vector_0000 TLBrefill
        ##
        .org x_EXCEPTION_0000,0
_excp_0000:
	la   $k0, HW_dsp7seg_addr  	# 7 segment display
	li   $k1, 0x0399		# display .9.9
	sw   $k1, 0($k0)		# write to 7 segment display
h0000:	j    h0000			# wait forever
	nop


        ##
        ##================================================================
        ## exception vector_0100 Cache Error (hw not implemented)
        ##   print CAUSE and stop simulation
        ##
         .org x_EXCEPTION_0100,0
_excp_0100:
	la   $k0, HW_dsp7seg_addr  	# 7 segment display
	li   $k1, 0x0388		# display .8.8
	sw   $k1, 0($k0)		# write to 7 segment display
h0100:	j    h0100			# wait forever
	nop


        ##
        ##================================================================
        ## handler for all exceptions except interrupts and TLBrefill
        ##
        .org x_EXCEPTION_0180,0
_excp_0180:
	la   $k0, HW_dsp7seg_addr  	# 7 segment display
	li   $k1, 0x0377		# display .7.7
	sw   $k1, 0($k0)		# write to 7 segment display
h0180:	j    h0180			# wait forever
	nop


        ##
        ##===============================================================
        ## interrupt handlers at exception vector 0200
        ##
        .org x_EXCEPTION_0200,0
_excp_0200:
	la   $k0, HW_dsp7seg_addr  	# 7 segment display
	li   $k1, 0x0366		# display .6.6
	sw   $k1, 0($k0)		# write to 7 segment display
h0200:	j    h0200			# wait forever
	nop
	

        ##
        ##================================================================
        ## exception vector_BFC0 NMI or soft-reset
        ##
	.org x_EXCEPTION_BFC0,0
_excp_BFC0:
	la   $k0, HW_dsp7seg_addr  	# 7 segment display
	li   $k1, 0x0355		# display .5.5
	sw   $k1, 0($k0)		# write to 7 segment display
hBFC0:	j    hBFC0			# wait forever
	nop

	##================================================================

	##
	##===============================================================
	## main(), normal code starts below -- do not edit next line
	.org x_ENTRY_POINT,0

